{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math, sysutils;

PROCEDURE counts;
PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;
  newyear:Boolean = false;
  DayofYear: double = 0;
  h: array[1..4,1..4] of double;

implementation

uses frontend, calculate, options;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'Mel-like';
ModelDef.versionnumber := '1.0.0';
ModelDef.timeunit := 'days';
ModelDef.contactperson := 'Chelsea';
ModelDef.contactaddress1 := 'Ecosystems Center';
ModelDef.contactaddress2 := 'Marine Biological Laboratory';
ModelDef.contactaddress3 := 'Woods Hole, MA 02536';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 61;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is stringlength (specified in unit stypes) characters. }
 
 
with stat[1] do
 begin
    name:='Biomass C';  units:='gC m-2';  symbol:='BC';
 end;
 
with stat[2] do
 begin
    name:='Biomass N';  units:='gN m-2';  symbol:='BN';
 end;
 
with stat[3] do
 begin
    name:='Biomass P';  units:='gP m-2';  symbol:='BP';
 end;
 
with stat[4] do
 begin
    name:='Effort C';  units:='fraction';  symbol:='VC';
 end;
 
with stat[5] do
 begin
    name:='Effort N';  units:='fraction';  symbol:='VN';
 end;
 
with stat[6] do
 begin
    name:='Effort P';  units:='fraction';  symbol:='VP';
 end;
 
with stat[7] do
 begin
    name:='Course woody debris C';  units:='gC m-2';  symbol:='CWDC';
 end;
 
with stat[8] do
 begin
    name:='Course woody debris N';  units:='gN m-2';  symbol:='CWDN';
 end;
 
with stat[9] do
 begin
    name:='Course woody debris P';  units:='gP m-2';  symbol:='CWDP';
 end;
 
with stat[10] do
 begin
    name:='Int uptake C';  units:='gC m-2';  symbol:='IUC';
 end;
 
with stat[11] do
 begin
    name:='Int uptake N';  units:='gN m-2';  symbol:='IUN';
 end;
 
with stat[12] do
 begin
    name:='Int uptake P';  units:='gP m-2';  symbol:='IUP';
 end;
 
with stat[13] do
 begin
    name:='Int req C';  units:='gC m-2';  symbol:='IRC';
 end;
 
with stat[14] do
 begin
    name:='Int req N';  units:='gN m-2';  symbol:='IRN';
 end;
 
with stat[15] do
 begin
    name:='Int req P';  units:='gP m-2';  symbol:='IRP';
 end;
 
with stat[16] do
 begin
    name:='cum UN';  units:='gN m-2';  symbol:='cumUN';
 end;
 
with stat[17] do
 begin
    name:='cum UP';  units:='gP m-2';  symbol:='cumUP';
 end;
 
with stat[18] do
 begin
    name:='cum GPP';  units:='gC m-2';  symbol:='cumGPP';
 end;
 
with stat[19] do
 begin
    name:='cum Rat';  units:='gC m-2';  symbol:='cumRat';
 end;
 
with stat[20] do
 begin
    name:='cum Ram';  units:='gC m-2';  symbol:='cumRam';
 end;
 
with stat[21] do
 begin
    name:='cum Rag';  units:='gC m-2';  symbol:='cumRag';
 end;
 
with stat[22] do
 begin
    name:='cum NPP';  units:='gC m-2';  symbol:='cumNPP';
 end;
 
with stat[23] do
 begin
    name:='cum LFC';  units:='gC m-2';  symbol:='cumLFC';
 end;
 
with stat[24] do
 begin
    name:='cum LFN';  units:='gN m-2';  symbol:='cumLFN';
 end;
 
with stat[25] do
 begin
    name:='cum LFP';  units:='gP m-2';  symbol:='cumLFP';
 end;
 
with stat[26] do
 begin
    name:='cum WDC';  units:='gC m-2';  symbol:='cumWDC';
 end;
 
with stat[27] do
 begin
    name:='cum WDN';  units:='gN m-2';  symbol:='cumWDN';
 end;
 
with stat[28] do
 begin
    name:='cum WDP';  units:='gP m-2';  symbol:='cumWDP';
 end;
 
with stat[29] do
 begin
    name:='cum WTC';  units:='gC m-2';  symbol:='cumWTC';
 end;
 
with stat[30] do
 begin
    name:='cum WTN';  units:='gN m-2';  symbol:='cumWTN';
 end;
 
with stat[31] do
 begin
    name:='cum WTP';  units:='gP m-2';  symbol:='cumWTP';
 end;
 
with stat[32] do
 begin
    name:='cum Nfix';  units:='gN m-2';  symbol:='cumNfix';
 end;
 
with stat[33] do
 begin
    name:='Inorganic C';  units:='gC m-2';  symbol:='IOC';
 end;
 
with stat[34] do
 begin
    name:='Total Inorganic N';  units:='gN m-2';  symbol:='TION';
 end;
 
with stat[35] do
 begin
    name:='Total Inorganic P';  units:='gP m-2';  symbol:='TIOP';
 end;
 
with stat[36] do
 begin
    name:='Phase I Soil organic C';  units:='gC m-2';  symbol:='SOMCI';
 end;
 
with stat[37] do
 begin
    name:='Phase I Soil organic N';  units:='gN m-2';  symbol:='SOMNI';
 end;
 
with stat[38] do
 begin
    name:='Phase I Soil organic P';  units:='gP m-2';  symbol:='SOMPI';
 end;
 
with stat[39] do
 begin
    name:='Phase II Soil organic C';  units:='gC m-2';  symbol:='SOMCII';
 end;
 
with stat[40] do
 begin
    name:='Phase II Soil organic N';  units:='gN m-2';  symbol:='SOMNII';
 end;
 
with stat[41] do
 begin
    name:='Phase II Soil organic P';  units:='gP m-2';  symbol:='SOMPII';
 end;
 
with stat[42] do
 begin
    name:='cum DC';  units:='gC m-2';  symbol:='cumDC';
 end;
 
with stat[43] do
 begin
    name:='cum DN';  units:='gN m-2';  symbol:='cumDN';
 end;
 
with stat[44] do
 begin
    name:='cum DP';  units:='gP m-2';  symbol:='cumDP';
 end;
 
with stat[45] do
 begin
    name:='cum QC';  units:='gC m-2';  symbol:='cumQC';
 end;
 
with stat[46] do
 begin
    name:='cum QN';  units:='gN m-2';  symbol:='cumQN';
 end;
 
with stat[47] do
 begin
    name:='cum QP';  units:='gP m-2';  symbol:='cumQP';
 end;
 
with stat[48] do
 begin
    name:='cum QOCI';  units:='gC m-2';  symbol:='cumQOCI';
 end;
 
with stat[49] do
 begin
    name:='cum QONI';  units:='gN m-2';  symbol:='cumQONI';
 end;
 
with stat[50] do
 begin
    name:='cum QOPI';  units:='gP m-2';  symbol:='cumQOPI';
 end;
 
with stat[51] do
 begin
    name:='cum QOCII';  units:='gC m-2';  symbol:='cumQOCII';
 end;
 
with stat[52] do
 begin
    name:='cum QONII';  units:='gN m-2';  symbol:='cumQONII';
 end;
 
with stat[53] do
 begin
    name:='cum QOPII';  units:='gP m-2';  symbol:='cumQOPII';
 end;
 
with stat[54] do
 begin
    name:='cum RhI';  units:='gC m-2';  symbol:='cumRhI';
 end;
 
with stat[55] do
 begin
    name:='cum RhII';  units:='gC m-2';  symbol:='cumRhII';
 end;
 
with stat[56] do
 begin
    name:='cum NminI';  units:='gN m-2';  symbol:='cumNminI';
 end;
 
with stat[57] do
 begin
    name:='cum NminII';  units:='gN m-2';  symbol:='cumNminII';
 end;
 
with stat[58] do
 begin
    name:='cum PminI';  units:='gN m-2';  symbol:='cumPminI';
 end;
 
with stat[59] do
 begin
    name:='cum PminII';  units:='gN m-2';  symbol:='cumPminII';
 end;
 
with stat[60] do
 begin
    name:='cum IMMN';  units:='gN m-2';  symbol:='cumIMMN';
 end;
 
with stat[61] do
 begin
    name:='cum IMMP';  units:='gP m-2';  symbol:='cumIMMP';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 70;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'Gross prim prod';
      units       := 'gC m-2 d-1';
      symbol       := 'GPP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate of GPP';  units:='gC m-2 d-1';  symbol:='rGPP';
 end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'Auto resp meta';
      units       := 'gC m-2 d-1';
      symbol       := 'Ram';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Q10Ram';  units:='none';  symbol:='Q10Ram';
 end;
with par[npar + 2] do
 begin
    name:='Rate auto resp meta';  units:='gC m-2';  symbol:='rRam';
 end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'Auto resp growth';
      units       := 'gC m-2 d-1';
      symbol       := 'Rag';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate auto resp growth';  units:='gC m-2';  symbol:='rRag';
 end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'Total auto resp';
      units       := 'g C m-2 d-1';
      symbol       := 'Rat';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'Net prim prod';
      units       := 'gC m-2 d-1';
      symbol       := 'NPP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'Uptake N';
      units       := 'gN m-2 d-1';
      symbol       := 'UN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate N uptake';  units:='gN m-2';  symbol:='rUN';
 end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'Uptake P';
      units       := 'gP m-2 d-1';
      symbol       := 'UP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate P uptake';  units:='gP m-2';  symbol:='rUP';
 end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'Dum1';
      units       := 'none';
      symbol       := 'dum1';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Require const';  units:='none';  symbol:='sigma';
 end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'Dum2';
      units       := 'none';
      symbol       := 'dum2';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Acclim const';  units:='none';  symbol:='alpha';
 end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'oqcn';
      units       := 'none';
      symbol       := 'oqcn';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Dum3';  units:='none';  symbol:='dum3';
 end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'oqcp';
      units       := 'none';
      symbol       := 'oqcp';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Dum4';  units:='none';  symbol:='dum4';
 end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'Acclim adjust';
      units       := 'none';
      symbol       := 'phi';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'Acclim check C';
      units       := 'none';
      symbol       := 'LambdaC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'Acclim check N';
      units       := 'none';
      symbol       := 'LambdaN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'Acclim check P';
      units       := 'none';
      symbol       := 'LambdaP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := 'Litterfall C';
      units       := 'gC m-2 d-1';
      symbol       := 'LFC';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate LF C fast';  units:='gC m-2';  symbol:='rLFC';
 end;
with par[npar + 2] do
 begin
    name:='Rate LF C slow';  units:='gC m-2';  symbol:='rLFCs';
 end;
 
CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'Litterfall N';
      units       := 'gN m-2 d-1';
      symbol       := 'LFN';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate LF N fast';  units:='gN m-2';  symbol:='rLFN';
 end;
with par[npar + 2] do
 begin
    name:='Rate LF N slow';  units:='gN m-2';  symbol:='rLFNs';
 end;
 
CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'Litterfall P';
      units       := 'gP m-2 d-1';
      symbol       := 'LFP';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate LF P fast';  units:='gP m-2';  symbol:='rLFP';
 end;
with par[npar + 2] do
 begin
    name:='Rate LF P slow';  units:='gP m-2';  symbol:='rLFPs';
 end;
 
CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'Woody debris C prod';
      units       := 'gC m-2 d-1';
      symbol       := 'WDC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate WDC prod';  units:='d-1';  symbol:='rWDC';
 end;
 
CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'Woody debris N prod';
      units       := 'gN m-2 d-1';
      symbol       := 'WDN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C:N CWD';  units:='g C g-1N';  symbol:='qcnWD';
 end;
 
CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'Woody debris P prod';
      units       := 'gP m-2 d-1';
      symbol       := 'WDP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C:P CWD';  units:='g C g-1 P2';  symbol:='qcpWD';
 end;
 
CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen uptake cost';
      units       := 'gN m-2 d-1';
      symbol       := 'Nupcost';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N fix cost';  units:='gN m-2';  symbol:='Nfixcost';
 end;
 
CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen fixation';
      units       := 'gN m-2 d-1';
      symbol       := 'Nfix';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate Nfix';  units:='gN m-2';  symbol:='rNfix';
 end;
 
CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'Require C';
      units       := 'gC m-2 d-1';
      symbol       := 'RC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'Require N';
      units       := 'gN m-2 d-1';
      symbol       := 'RN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'Require P';
      units       := 'gP m-2 d-1';
      symbol       := 'RP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'Active biomass';
      units       := 'gC m-2';
      symbol       := 'BA';
      parameters       := 4;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='BA max';  units:='gC m-2';  symbol:='BAmax';
 end;
with par[npar + 2] do
 begin
    name:='Init slope';  units:='none';  symbol:='c';
 end;
with par[npar + 3] do
 begin
    name:='optimal c to n active';  units:='none';  symbol:='oqcnA';
 end;
with par[npar + 4] do
 begin
    name:='optimal c to p active';  units:='none';  symbol:='oqcpA';
 end;
 
CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'Woody biomass';
      units       := 'gC m-2';
      symbol       := 'BW';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='optimal c to n woody';  units:='none';  symbol:='oqcnW';
 end;
with par[npar + 2] do
 begin
    name:='optimal c to p woody';  units:='none';  symbol:='oqcpW';
 end;
 
CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'Leaf area';
      units       := 'm2 m-2';
      symbol       := 'LA';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Canopy dominance';  units:='none';  symbol:='gamma';
 end;
with par[npar + 2] do
 begin
    name:='Leaf mass area';  units:='g m-2';  symbol:='LMA';
 end;
 
CurrentProc := ModelDef.numstate + 30;
With proc[CurrentProc] do
   begin
      name       := 'Leaf area total';
      units       := 'm2 m-2';
      symbol       := 'LAT2';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Half sat CO2';  units:='mL m-3';  symbol:='kc';
 end;
 
CurrentProc := ModelDef.numstate + 31;
With proc[CurrentProc] do
   begin
      name       := 'Root length';
      units       := 'm m-1';
      symbol       := 'RL';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Below dominance';  units:='none';  symbol:='gammaR';
 end;
 
CurrentProc := ModelDef.numstate + 32;
With proc[CurrentProc] do
   begin
      name       := 'Dum5';
      units       := 'none';
      symbol       := 'dum5';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Half sat N';  units:='mL m-3';  symbol:='kn';
 end;
 
CurrentProc := ModelDef.numstate + 33;
With proc[CurrentProc] do
   begin
      name       := 'Dum6';
      units       := 'none';
      symbol       := 'dum6';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Half sat P';  units:='mL m-3';  symbol:='kp';
 end;
 
CurrentProc := ModelDef.numstate + 34;
With proc[CurrentProc] do
   begin
      name       := 'Woody turnover C';
      units       := 'gC m-2 d-1';
      symbol       := 'WTC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate woody turnover';  units:='gC m-2';  symbol:='rWTC';
 end;
 
CurrentProc := ModelDef.numstate + 35;
With proc[CurrentProc] do
   begin
      name       := 'Woody turnover N';
      units       := 'gN m-2 d-1';
      symbol       := 'WTN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Q10Ps';  units:='none';  symbol:='Q10Ps';
 end;
 
CurrentProc := ModelDef.numstate + 36;
With proc[CurrentProc] do
   begin
      name       := 'Woody turnover P';
      units       := 'gP m-2 d-1';
      symbol       := 'WTP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 37;
With proc[CurrentProc] do
   begin
      name       := 'PhaseI to PhaseII C';
      units       := 'gC m-2 d-1';
      symbol       := 'DC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate PhaseI to PhaseII C';  units:='gC m-2';  symbol:='rDC';
 end;
 
CurrentProc := ModelDef.numstate + 38;
With proc[CurrentProc] do
   begin
      name       := 'PhaseI to PhaseII N';
      units       := 'gN m-2 d-1';
      symbol       := 'DN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='PhaseII C:N';  units:='g C g-1 N';  symbol:='CNPhII';
 end;
 
CurrentProc := ModelDef.numstate + 39;
With proc[CurrentProc] do
   begin
      name       := 'PhaseI to PhaseII P';
      units       := 'gP m-2 d-1';
      symbol       := 'DP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='PhaseII C:P';  units:='g C g-1 P';  symbol:='CPPhII';
 end;
 
CurrentProc := ModelDef.numstate + 40;
With proc[CurrentProc] do
   begin
      name       := 'Inorganic loss C';
      units       := 'gC m-2 d-1';
      symbol       := 'QC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QC';  units:='gC m-2';  symbol:='rQC';
 end;
 
CurrentProc := ModelDef.numstate + 41;
With proc[CurrentProc] do
   begin
      name       := 'Inorganic loss N';
      units       := 'gN m-2 d-1';
      symbol       := 'QN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QN';  units:='gN m-2';  symbol:='rQN';
 end;
 
CurrentProc := ModelDef.numstate + 42;
With proc[CurrentProc] do
   begin
      name       := 'Inorganic loss P';
      units       := 'gP m-2 d-1';
      symbol       := 'QP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QP';  units:='gP m-2';  symbol:='rQP';
 end;
 
CurrentProc := ModelDef.numstate + 43;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss CI';
      units       := 'gC m-2 d-1';
      symbol       := 'QOCI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QOCI';  units:='gC m-2';  symbol:='rQOCI';
 end;
 
CurrentProc := ModelDef.numstate + 44;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss NI';
      units       := 'gN m-2 d-1';
      symbol       := 'QONI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QONI';  units:='gN m-2';  symbol:='rQONI';
 end;
 
CurrentProc := ModelDef.numstate + 45;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss PI';
      units       := 'gP m-2 d-1';
      symbol       := 'QOPI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QOPI';  units:='gP m-2';  symbol:='rQOPI';
 end;
 
CurrentProc := ModelDef.numstate + 46;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss CII';
      units       := 'gC m-2 d-1';
      symbol       := 'QOCII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate QOCII';  units:='gC m-2';  symbol:='rQOCII';
 end;
 
CurrentProc := ModelDef.numstate + 47;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss NII';
      units       := 'gN m-2 d-1';
      symbol       := 'QONII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Veg Calibrate';  units:='units';  symbol:='calveg';
 end;
 
CurrentProc := ModelDef.numstate + 48;
With proc[CurrentProc] do
   begin
      name       := 'Organic loss PII';
      units       := 'gP m-2 d-1';
      symbol       := 'QOPII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Q10soil';  units:='none';  symbol:='Q10soil';
 end;
 
CurrentProc := ModelDef.numstate + 49;
With proc[CurrentProc] do
   begin
      name       := 'Hetero phaseI resp';
      units       := 'gC m-2 d-1';
      symbol       := 'RhI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate hetero phaseI resp';  units:='gC m-2';  symbol:='rRhI';
 end;
 
CurrentProc := ModelDef.numstate + 50;
With proc[CurrentProc] do
   begin
      name       := 'Hetero phaseII resp';
      units       := 'gC m-2 d-1';
      symbol       := 'RhII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate hetero phaseII resp';  units:='gC m-2';  symbol:='rRhII';
 end;
 
CurrentProc := ModelDef.numstate + 51;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen minI';
      units       := 'gN m-2 d-1';
      symbol       := 'NminI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate N minI';  units:='gN m-2';  symbol:='rNminI';
 end;
 
CurrentProc := ModelDef.numstate + 52;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen minII';
      units       := 'gN m-2 d-1';
      symbol       := 'NminII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Cal N fix';  units:='none';  symbol:='calNfix';
 end;
 
CurrentProc := ModelDef.numstate + 53;
With proc[CurrentProc] do
   begin
      name       := 'Phosphorus minI';
      units       := 'gP m-2 d-1';
      symbol       := 'PminI';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Rate P minI';  units:='gP m-2';  symbol:='rPminI';
 end;
 
CurrentProc := ModelDef.numstate + 54;
With proc[CurrentProc] do
   begin
      name       := 'Phosphorus minII';
      units       := 'gP m-2 d-1';
      symbol       := 'PminII';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Calibrator exponent';  units:='none';  symbol:='CALEXP';
 end;
 
CurrentProc := ModelDef.numstate + 55;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen immob';
      units       := 'gN m-2 d-1';
      symbol       := 'IMMN';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max N immob';  units:='gN m-2 d-1';  symbol:='IMMNmax';
 end;
with par[npar + 2] do
 begin
    name:='Half sat N immob';  units:='gN m-2';  symbol:='kIMMN';
 end;
 
CurrentProc := ModelDef.numstate + 56;
With proc[CurrentProc] do
   begin
      name       := 'Phosphorus immob';
      units       := 'gP m-2 d-1';
      symbol       := 'IMMP';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max P immob';  units:='gP m-2 d-1';  symbol:='IMMPmax';
 end;
with par[npar + 2] do
 begin
    name:='Half sat P immob';  units:='gP m-2';  symbol:='kIMMP';
 end;
 
CurrentProc := ModelDef.numstate + 57;
With proc[CurrentProc] do
   begin
      name       := 'Solution N';
      units       := 'umol L-1';
      symbol       := 'ION';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max N sorbed';  units:='gN m-2';  symbol:='SNmax';
 end;
with par[npar + 2] do
 begin
    name:='Sorbed N half sat';  units:='gN m-2';  symbol:='kNsorp';
 end;
 
CurrentProc := ModelDef.numstate + 58;
With proc[CurrentProc] do
   begin
      name       := 'Solution P';
      units       := 'g P m-2';
      symbol       := 'IOP';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max P sorbed';  units:='gP m-2';  symbol:='SPmax';
 end;
with par[npar + 2] do
 begin
    name:='Sorbed P half sat';  units:='gP m-2';  symbol:='kPsorp';
 end;
 
CurrentProc := ModelDef.numstate + 59;
With proc[CurrentProc] do
   begin
      name       := 'Julian Day';
      units       := 'days';
      symbol       := 'Jday';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='J day divider';  units:='days';  symbol:='JdayD';
 end;
 
CurrentProc := ModelDef.numstate + 60;
With proc[CurrentProc] do
   begin
      name       := 'soil temperature';
      units       := 'oC';
      symbol       := 'Tsoil';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 61;
With proc[CurrentProc] do
   begin
      name       := 'soil metab index';
      units       := 'none';
      symbol       := 'SMI';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 62;
With proc[CurrentProc] do
   begin
      name       := 'runoff index';
      units       := 'none';
      symbol       := 'ROI';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 63;
With proc[CurrentProc] do
   begin
      name       := 'Calibrate';
      units       := 'units';
      symbol       := 'calib';
      parameters       := 21;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Calibrate';  units:='units';  symbol:='cal';
 end;
with par[npar + 2] do
 begin
    name:='Target DC';  units:='gC m-2';  symbol:='tDC';
 end;
with par[npar + 3] do
 begin
    name:='Target DN';  units:='gN m-2';  symbol:='tDN';
 end;
with par[npar + 4] do
 begin
    name:='Target DP';  units:='gP m-2';  symbol:='tDP';
 end;
with par[npar + 5] do
 begin
    name:='Target QC';  units:='gC m-2';  symbol:='tQC';
 end;
with par[npar + 6] do
 begin
    name:='Target QN';  units:='gN m-2';  symbol:='tQN';
 end;
with par[npar + 7] do
 begin
    name:='Target QP';  units:='gP m-2';  symbol:='tQP';
 end;
with par[npar + 8] do
 begin
    name:='Target QOCI';  units:='gC m-2';  symbol:='tQOCI';
 end;
with par[npar + 9] do
 begin
    name:='Target QONI';  units:='gN m-2';  symbol:='tQONI';
 end;
with par[npar + 10] do
 begin
    name:='Target QOPI';  units:='gP m-2';  symbol:='tQOPI';
 end;
with par[npar + 11] do
 begin
    name:='Target QOCII';  units:='gC m-2';  symbol:='tQOCII';
 end;
with par[npar + 12] do
 begin
    name:='Target QONII';  units:='gN m-2';  symbol:='tQONII';
 end;
with par[npar + 13] do
 begin
    name:='Target QOPII';  units:='gP m-2';  symbol:='tQOPII';
 end;
with par[npar + 14] do
 begin
    name:='Target RhI';  units:='gC m-2';  symbol:='tRhI';
 end;
with par[npar + 15] do
 begin
    name:='Target RhII';  units:='gC m-2';  symbol:='tRhII';
 end;
with par[npar + 16] do
 begin
    name:='Target NminI';  units:='gN m-2';  symbol:='tNminI';
 end;
with par[npar + 17] do
 begin
    name:='Target NminII';  units:='gN m-2';  symbol:='tNminII';
 end;
with par[npar + 18] do
 begin
    name:='Target PminI';  units:='gP m-2';  symbol:='tPminI';
 end;
with par[npar + 19] do
 begin
    name:='Target PminII';  units:='gP m-2';  symbol:='tPminII';
 end;
with par[npar + 20] do
 begin
    name:='Target IMMN';  units:='gN m-2';  symbol:='tIMMN';
 end;
with par[npar + 21] do
 begin
    name:='Target IMMP';  units:='gP m-2';  symbol:='tIMMP';
 end;
 
CurrentProc := ModelDef.numstate + 64;
With proc[CurrentProc] do
   begin
      name       := 'CalibrateA';
      units       := 'units';
      symbol       := 'calibA';
      parameters       := 10;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Target GPP';  units:='gC m-2';  symbol:='tGPP';
 end;
with par[npar + 2] do
 begin
    name:='Target Rat';  units:='gC m-2';  symbol:='tRat';
 end;
with par[npar + 3] do
 begin
    name:='Target LFC';  units:='gC m-2';  symbol:='tLFC';
 end;
with par[npar + 4] do
 begin
    name:='Target WDC';  units:='gC m-2';  symbol:='tWDC';
 end;
with par[npar + 5] do
 begin
    name:='Target NPP';  units:='gC m-2';  symbol:='tNPP';
 end;
with par[npar + 6] do
 begin
    name:='Target UN';  units:='gN m-2';  symbol:='tUN';
 end;
with par[npar + 7] do
 begin
    name:='Target UP';  units:='gP m-2';  symbol:='tUP';
 end;
with par[npar + 8] do
 begin
    name:='Target LFN';  units:='gN m-2';  symbol:='tLFN';
 end;
with par[npar + 9] do
 begin
    name:='Target LFP';  units:='gP m-2';  symbol:='tLFP';
 end;
with par[npar + 10] do
 begin
    name:='Target WDN';  units:='gN m-2';  symbol:='tWDN';
 end;
 
CurrentProc := ModelDef.numstate + 65;
With proc[CurrentProc] do
   begin
      name       := 'CalibrateB';
      units       := 'units';
      symbol       := 'calibB';
      parameters       := 10;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Target WDP';  units:='gP m-2';  symbol:='tWDP';
 end;
with par[npar + 2] do
 begin
    name:='Target WTC';  units:='gC m-2';  symbol:='tWTC';
 end;
with par[npar + 3] do
 begin
    name:='Target WTN';  units:='gN m-2';  symbol:='tWTN';
 end;
with par[npar + 4] do
 begin
    name:='Target WTP';  units:='gP m-2';  symbol:='tWTP';
 end;
with par[npar + 5] do
 begin
    name:='Target Ram';  units:='gC m-2';  symbol:='tRam';
 end;
with par[npar + 6] do
 begin
    name:='Target Rag';  units:='gC m-2';  symbol:='tRag';
 end;
with par[npar + 7] do
 begin
    name:='Target Nfix';  units:='gN m-2';  symbol:='tNfix';
 end;
with par[npar + 8] do
 begin
    name:='Target VC';  units:='fraction';  symbol:='tVC';
 end;
with par[npar + 9] do
 begin
    name:='Target VN';  units:='fraction';  symbol:='tVN';
 end;
with par[npar + 10] do
 begin
    name:='Target VP';  units:='fraction';  symbol:='tVP';
 end;
 
CurrentProc := ModelDef.numstate + 66;
With proc[CurrentProc] do
   begin
      name       := 'Actual CN';
      units       := 'none';
      symbol       := 'aqcn';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 67;
With proc[CurrentProc] do
   begin
      name       := 'Actual CP';
      units       := 'none';
      symbol       := 'aqcp';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 68;
With proc[CurrentProc] do
   begin
      name       := 'Net ecosys prod';
      units       := 'gC m-2 d-1';
      symbol       := 'NEP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 69;
With proc[CurrentProc] do
   begin
      name       := 'Hetero resp total';
      units       := 'gC m-2 d-1';
      symbol       := 'Rht';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 70;
With proc[CurrentProc] do
   begin
      name       := 'Ecosys resp';
      units       := 'gC m-2 d-1';
      symbol       := 'Re';
      parameters       := 0;
      ptype       := ptGroup1;
   end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 9;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='Temperature';  units:='degrees';  symbol:='T';
 end;
 
with drive[2] do
 begin
    name:='Carbon dioxide';  units:='ppm';  symbol:='CO2';
 end;
 
with drive[3] do
 begin
    name:='Nitrogen dep';  units:='gN m-2 d-1';  symbol:='Ndep';
 end;
 
with drive[4] do
 begin
    name:='Phos weathering';  units:='gP m-2 d-1';  symbol:='PW';
 end;
 
with drive[5] do
 begin
    name:='Top canopy irr';  units:='umol m-2 s-1';  symbol:='I0';
 end;
 
with drive[6] do
 begin
    name:='Immigration';  units:='gC m-2 d-1';  symbol:='IMG';
 end;
 
with drive[7] do
 begin
    name:='Manure C add';  units:='gC m-2 d-1';  symbol:='ManC';
 end;
 
with drive[8] do
 begin
    name:='Manure N add';  units:='gN m-2 d-1';  symbol:='ManN';
 end;
 
with drive[9] do
 begin
    name:='Manure P add';  units:='gP m-2 d-1';  symbol:='ManP';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

// Choose either GlobalPs, ArcticPs, or none here so the appropriate Ps model is compiled below.
{$DEFINE none}

PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
{$IFDEF GlobalPs}
const
// Global Ps parameters
 x1 = 11.04;             x2 = 0.03;
 x5 = 0.216;             x6 = 0.6;
 x7 = 3.332;             x8 = 0.004;
 x9 = 1.549;             x10 = 1.156;
 gammastar = 0;          kCO2 = 995.4;  }
{$ENDIF}

// Modify constant above (line above "procedure processes..." line )to specify
// which Ps model and it's constants should be compiled. Choosing a Ps model
// automatically includes the Et and Misc constants (i.e. Gem is assumed).

{$IFDEF ArcticPs}
const
// Arctic Ps parameters
x1 = 0.192;	x2 = 0.125;
x5 = 2.196;	x6 = 50.41;
x7 = 0.161;	x8 = 14.78;
x9 = 1.146;
gammastar = 0.468;	kCO2 = 500.3;
{$ENDIF}

{$IFDEF ArcticPs OR GlobalPs}
//const
// General Et parameters
aE1 = 0.0004;    aE2 = 150;  aE3 = 1.21;   aE4 = 6.11262E5;

// Other constants
cp = 1.012E-9; //specific heat air MJ kg-1 oC-1
sigmaSB = 4.9e-9; //stefan-Boltzmann MJ m-2 day-1 K-4
S0 = 117.5; //solar constant MJ m-2 day-1
bHI1 =0.23;
bHI2 =0.48;
mw = 2.99; //kg h2o MJ-1
alphaMS = 2; //mm oC-1 day-1                                 }
{$ENDIF}

var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
BC, dBCdt, 
BN, dBNdt, 
BP, dBPdt, 
VC, dVCdt, 
VN, dVNdt, 
VP, dVPdt, 
CWDC, dCWDCdt, 
CWDN, dCWDNdt, 
CWDP, dCWDPdt, 
IUC, dIUCdt, 
IUN, dIUNdt, 
IUP, dIUPdt, 
IRC, dIRCdt, 
IRN, dIRNdt, 
IRP, dIRPdt, 
cumUN, dcumUNdt, 
cumUP, dcumUPdt, 
cumGPP, dcumGPPdt, 
cumRat, dcumRatdt, 
cumRam, dcumRamdt, 
cumRag, dcumRagdt, 
cumNPP, dcumNPPdt, 
cumLFC, dcumLFCdt, 
cumLFN, dcumLFNdt, 
cumLFP, dcumLFPdt, 
cumWDC, dcumWDCdt, 
cumWDN, dcumWDNdt, 
cumWDP, dcumWDPdt, 
cumWTC, dcumWTCdt, 
cumWTN, dcumWTNdt, 
cumWTP, dcumWTPdt, 
cumNfix, dcumNfixdt, 
IOC, dIOCdt, 
TION, dTIONdt, 
TIOP, dTIOPdt, 
SOMCI, dSOMCIdt, 
SOMNI, dSOMNIdt, 
SOMPI, dSOMPIdt, 
SOMCII, dSOMCIIdt, 
SOMNII, dSOMNIIdt, 
SOMPII, dSOMPIIdt, 
cumDC, dcumDCdt, 
cumDN, dcumDNdt, 
cumDP, dcumDPdt, 
cumQC, dcumQCdt, 
cumQN, dcumQNdt, 
cumQP, dcumQPdt, 
cumQOCI, dcumQOCIdt, 
cumQONI, dcumQONIdt, 
cumQOPI, dcumQOPIdt, 
cumQOCII, dcumQOCIIdt, 
cumQONII, dcumQONIIdt, 
cumQOPII, dcumQOPIIdt, 
cumRhI, dcumRhIdt, 
cumRhII, dcumRhIIdt, 
cumNminI, dcumNminIdt, 
cumNminII, dcumNminIIdt, 
cumPminI, dcumPminIdt, 
cumPminII, dcumPminIIdt, 
cumIMMN, dcumIMMNdt, 
cumIMMP, dcumIMMPdt, 

{processes and associated parameters}
GPP, rGPP, 
Ram, Q10Ram, rRam, 
Rag, rRag, 
Rat, 
NPP, 
UN, rUN, 
UP, rUP, 
dum1, sigma, 
dum2, alpha, 
oqcn, dum3, 
oqcp, dum4, 
phi, 
LambdaC, 
LambdaN, 
LambdaP, 
LFC, rLFC, rLFCs, 
LFN, rLFN, rLFNs, 
LFP, rLFP, rLFPs, 
WDC, rWDC, 
WDN, qcnWD, 
WDP, qcpWD, 
Nupcost, Nfixcost, 
Nfix, rNfix, 
RC, 
RN, 
RP, 
BA, BAmax, c, oqcnA, oqcpA, 
BW, oqcnW, oqcpW, 
LA, gamma, LMA, 
LAT2, kc, 
RL, gammaR, 
dum5, kn, 
dum6, kp, 
WTC, rWTC, 
WTN, Q10Ps, 
WTP, 
DC, rDC, 
DN, CNPhII, 
DP, CPPhII, 
QC, rQC, 
QN, rQN, 
QP, rQP, 
QOCI, rQOCI, 
QONI, rQONI, 
QOPI, rQOPI, 
QOCII, rQOCII, 
QONII, calveg, 
QOPII, Q10soil, 
RhI, rRhI, 
RhII, rRhII, 
NminI, rNminI, 
NminII, calNfix, 
PminI, rPminI, 
PminII, CALEXP, 
IMMN, IMMNmax, kIMMN, 
IMMP, IMMPmax, kIMMP, 
ION, SNmax, kNsorp, 
IOP, SPmax, kPsorp, 
Jday, JdayD, 
Tsoil, 
SMI, 
ROI, 
calib, cal, tDC, tDN, tDP, tQC, tQN, tQP, tQOCI, tQONI, tQOPI, tQOCII, tQONII, tQOPII, tRhI, tRhII, tNminI, tNminII, tPminI, tPminII, tIMMN, tIMMP, 
calibA, tGPP, tRat, tLFC, tWDC, tNPP, tUN, tUP, tLFN, tLFP, tWDN, 
calibB, tWDP, tWTC, tWTN, tWTP, tRam, tRag, tNfix, tVC, tVN, tVP, 
aqcn, 
aqcp, 
NEP, 
Rht, 
Re, 

{drivers}
T, 
CO2, 
Ndep, 
PW, 
I0, 
IMG, 
ManC, 
ManN, 
ManP, 

{Other double}
dum, 
dumm, 
BWT

:double; {Final double}

{Other integers}
npar, j, jj, kk, ll, tnum:integer;

{ Boolean Variables }


{ Functions or procedures }

begin
{ Copy the drivers from the global array, drive, into the local variables. }
T := tdrive[1].value;
CO2 := tdrive[2].value;
Ndep := tdrive[3].value;
PW := tdrive[4].value;
I0 := tdrive[5].value;
IMG := tdrive[6].value;
ManC := tdrive[7].value;
ManN := tdrive[8].value;
ManP := tdrive[9].value;

{ Copy the state variables from the global array into the local variables. }
BC := tstat[1].value;
BN := tstat[2].value;
BP := tstat[3].value;
VC := tstat[4].value;
VN := tstat[5].value;
VP := tstat[6].value;
CWDC := tstat[7].value;
CWDN := tstat[8].value;
CWDP := tstat[9].value;
IUC := tstat[10].value;
IUN := tstat[11].value;
IUP := tstat[12].value;
IRC := tstat[13].value;
IRN := tstat[14].value;
IRP := tstat[15].value;
cumUN := tstat[16].value;
cumUP := tstat[17].value;
cumGPP := tstat[18].value;
cumRat := tstat[19].value;
cumRam := tstat[20].value;
cumRag := tstat[21].value;
cumNPP := tstat[22].value;
cumLFC := tstat[23].value;
cumLFN := tstat[24].value;
cumLFP := tstat[25].value;
cumWDC := tstat[26].value;
cumWDN := tstat[27].value;
cumWDP := tstat[28].value;
cumWTC := tstat[29].value;
cumWTN := tstat[30].value;
cumWTP := tstat[31].value;
cumNfix := tstat[32].value;
IOC := tstat[33].value;
TION := tstat[34].value;
TIOP := tstat[35].value;
SOMCI := tstat[36].value;
SOMNI := tstat[37].value;
SOMPI := tstat[38].value;
SOMCII := tstat[39].value;
SOMNII := tstat[40].value;
SOMPII := tstat[41].value;
cumDC := tstat[42].value;
cumDN := tstat[43].value;
cumDP := tstat[44].value;
cumQC := tstat[45].value;
cumQN := tstat[46].value;
cumQP := tstat[47].value;
cumQOCI := tstat[48].value;
cumQONI := tstat[49].value;
cumQOPI := tstat[50].value;
cumQOCII := tstat[51].value;
cumQONII := tstat[52].value;
cumQOPII := tstat[53].value;
cumRhI := tstat[54].value;
cumRhII := tstat[55].value;
cumNminI := tstat[56].value;
cumNminII := tstat[57].value;
cumPminI := tstat[58].value;
cumPminII := tstat[59].value;
cumIMMN := tstat[60].value;
cumIMMP := tstat[61].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 1);
rGPP := par[npar + 1].value;

npar:=ParCount(ModelDef.numstate + 2);
Q10Ram := par[npar + 1].value;
rRam := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 3);
rRag := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 6);
rUN := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 7);
rUP := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 8);
sigma := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 9);
alpha := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 10);
dum3 := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 11);
dum4 := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 16);
rLFC := par[npar + 1].value;
rLFCs := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 17);
rLFN := par[npar + 1].value;
rLFNs := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 18);
rLFP := par[npar + 1].value;
rLFPs := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 19);
rWDC := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 20);
qcnWD := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 21);
qcpWD := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 22);
Nfixcost := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 23);
rNfix := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 27);
BAmax := par[npar + 1].value;
c := par[npar + 2].value;
oqcnA := par[npar + 3].value;
oqcpA := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 28);
oqcnW := par[npar + 1].value;
oqcpW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 29);
gamma := par[npar + 1].value;
LMA := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 30);
kc := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 31);
gammaR := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 32);
kn := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 33);
kp := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 34);
rWTC := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 35);
Q10Ps := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 37);
rDC := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 38);
CNPhII := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 39);
CPPhII := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 40);
rQC := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 41);
rQN := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 42);
rQP := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 43);
rQOCI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 44);
rQONI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 45);
rQOPI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 46);
rQOCII := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 47);
calveg := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 48);
Q10soil := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 49);
rRhI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 50);
rRhII := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 51);
rNminI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 52);
calNfix := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 53);
rPminI := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 54);
CALEXP := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 55);
IMMNmax := par[npar + 1].value;
kIMMN := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 56);
IMMPmax := par[npar + 1].value;
kIMMP := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 57);
SNmax := par[npar + 1].value;
kNsorp := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 58);
SPmax := par[npar + 1].value;
kPsorp := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 59);
JdayD := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 63);
cal := par[npar + 1].value;
tDC := par[npar + 2].value;
tDN := par[npar + 3].value;
tDP := par[npar + 4].value;
tQC := par[npar + 5].value;
tQN := par[npar + 6].value;
tQP := par[npar + 7].value;
tQOCI := par[npar + 8].value;
tQONI := par[npar + 9].value;
tQOPI := par[npar + 10].value;
tQOCII := par[npar + 11].value;
tQONII := par[npar + 12].value;
tQOPII := par[npar + 13].value;
tRhI := par[npar + 14].value;
tRhII := par[npar + 15].value;
tNminI := par[npar + 16].value;
tNminII := par[npar + 17].value;
tPminI := par[npar + 18].value;
tPminII := par[npar + 19].value;
tIMMN := par[npar + 20].value;
tIMMP := par[npar + 21].value;
 
npar:=ParCount(ModelDef.numstate + 64);
tGPP := par[npar + 1].value;
tRat := par[npar + 2].value;
tLFC := par[npar + 3].value;
tWDC := par[npar + 4].value;
tNPP := par[npar + 5].value;
tUN := par[npar + 6].value;
tUP := par[npar + 7].value;
tLFN := par[npar + 8].value;
tLFP := par[npar + 9].value;
tWDN := par[npar + 10].value;
 
npar:=ParCount(ModelDef.numstate + 65);
tWDP := par[npar + 1].value;
tWTC := par[npar + 2].value;
tWTN := par[npar + 3].value;
tWTP := par[npar + 4].value;
tRam := par[npar + 5].value;
tRag := par[npar + 6].value;
tNfix := par[npar + 7].value;
tVC := par[npar + 8].value;
tVN := par[npar + 9].value;
tVP := par[npar + 10].value;
 
dBCdt := -999;
dBNdt := -999;
dBPdt := -999;
dVCdt := -999;
dVNdt := -999;
dVPdt := -999;
dCWDCdt := -999;
dCWDNdt := -999;
dCWDPdt := -999;
dIUCdt := -999;
dIUNdt := -999;
dIUPdt := -999;
dIRCdt := -999;
dIRNdt := -999;
dIRPdt := -999;
dcumUNdt := -999;
dcumUPdt := -999;
dcumGPPdt := -999;
dcumRatdt := -999;
dcumRamdt := -999;
dcumRagdt := -999;
dcumNPPdt := -999;
dcumLFCdt := -999;
dcumLFNdt := -999;
dcumLFPdt := -999;
dcumWDCdt := -999;
dcumWDNdt := -999;
dcumWDPdt := -999;
dcumWTCdt := -999;
dcumWTNdt := -999;
dcumWTPdt := -999;
dcumNfixdt := -999;
dIOCdt := -999;
dTIONdt := -999;
dTIOPdt := -999;
dSOMCIdt := -999;
dSOMNIdt := -999;
dSOMPIdt := -999;
dSOMCIIdt := -999;
dSOMNIIdt := -999;
dSOMPIIdt := -999;
dcumDCdt := -999;
dcumDNdt := -999;
dcumDPdt := -999;
dcumQCdt := -999;
dcumQNdt := -999;
dcumQPdt := -999;
dcumQOCIdt := -999;
dcumQONIdt := -999;
dcumQOPIdt := -999;
dcumQOCIIdt := -999;
dcumQONIIdt := -999;
dcumQOPIIdt := -999;
dcumRhIdt := -999;
dcumRhIIdt := -999;
dcumNminIdt := -999;
dcumNminIIdt := -999;
dcumPminIdt := -999;
dcumPminIIdt := -999;
dcumIMMNdt := -999;
dcumIMMPdt := -999;
GPP := -999;
Ram := -999;
Rag := -999;
Rat := -999;
NPP := -999;
UN := -999;
UP := -999;
dum1 := -999;
dum2 := -999;
oqcn := -999;
oqcp := -999;
phi := -999;
LambdaC := -999;
LambdaN := -999;
LambdaP := -999;
LFC := -999;
LFN := -999;
LFP := -999;
WDC := -999;
WDN := -999;
WDP := -999;
Nupcost := -999;
Nfix := -999;
RC := -999;
RN := -999;
RP := -999;
BA := -999;
BW := -999;
LA := -999;
LAT2 := -999;
RL := -999;
dum5 := -999;
dum6 := -999;
WTC := -999;
WTN := -999;
WTP := -999;
DC := -999;
DN := -999;
DP := -999;
QC := -999;
QN := -999;
QP := -999;
QOCI := -999;
QONI := -999;
QOPI := -999;
QOCII := -999;
QONII := -999;
QOPII := -999;
RhI := -999;
RhII := -999;
NminI := -999;
NminII := -999;
PminI := -999;
PminII := -999;
IMMN := -999;
IMMP := -999;
ION := -999;
IOP := -999;
Jday := -999;
Tsoil := -999;
SMI := -999;
ROI := -999;
calib := -999;
calibA := -999;
calibB := -999;
aqcn := -999;
aqcp := -999;
NEP := -999;
Rht := -999;
Re := -999;
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }


{Julian Day}
Jday:= tan(pi*((time)/JdayD -0.50001));
Jday:= arctan(Jday)+pi/2;
Jday:=trunc((JdayD/pi)*Jday+0.9);

    Dum:=VC+VN+VP;
    VC:=VC/dum; 
    VN:=VN/dum; 
    VP:=VP/dum;
  

if TION<0 then TION:=0;
if TIOP<0 then TIOP:=0;

     if BN<1e-6 then BN:=1e-6;
     if BN>BC then BN:=BC;
     if BP<1e-6 then BP:=1e-6; 
     if BP>BC then BP:=BC;
     if BC<=0 then 
       Begin
          BC:=0;
          BN:=0;
          BP:=0;
       End;
 
    if CWDC < 0 then CWDC:=0;
    if CWDN < 0 then CWDN:=0;
    if CWDP < 0 then CWDP:=0;


if SOMCI <1e-6 then SOMCI:= 1e-6;
if SOMNI <1e-6 then SOMNI := 1e-6;
if SOMPI <1e-6 then SOMPI:= 1e-6;
if SOMCII<1e-6 then SOMCII:= 1e-6;
if SOMNII<1e-6 then SOMNII := 1e-6;
if SOMPII<1e-6 then SOMPII:= 1e-6;
 

BWT:=0;

ION:=(TION-kNsorp-SNmax+sqrt(sqr(TION-kNsorp-SNmax)+4*TION*kNsorp))/2;
IOP:=(TIOP-kPsorp-SPmax+sqrt(sqr(TIOP-kPsorp-SPmax)+4*TIOP*kPsorp))/2;

        Dum1:=0;
         Dum2:=0;
         Dum5:=0;
         Dum6:=0;
         BA:=0;
           
                 
        BA:=BAmax*c*BC/(BAmax+c*BC);
        BW:=BC-BA;
  
   
   if BA>0 then
   begin  oqcn:=BC/((BA/oqcnA)+(BW/oqcnW));
          oqcp:=BC/((BA/oqcpA)+(BW/oqcpW));
   end
   else   begin oqcn:=1; oqcp:=0.1;
          end;
   if BA>0 then
   begin    aqcn:=BC/BN;
           aqcp:=BC/BP;
   end
   else 
        begin aqcn:=1; aqcp:=0.1;
        end;
   LA:=(VC*BA)/LMA;
       
    
        RL:=(VN+VP)*BA;
      
 
        Nupcost:= LMA*rUN*ION*(KC+CO2);

If Nupcost=0 then Nupcost:=1
Else Nupcost:=(0.5*rGPP*CO2*(KN+ION))/(Nupcost);
     Nupcost:=Nupcost*I0*exp((-0.5*VC*BA)/LMA);

      If Nupcost>Nfixcost then 
           Nfix:=rNfix*(Nupcost-Nfixcost)*(VN+VP)*BA 
       Else Nfix:=0;


         GPP:=rGPP*CO2*I0*(1-exp(-0.5*LA))/(kc+CO2);
   GPP:=GPP*power(Q10Ps,T/10);
Ram:=rRam*BN *power(Q10Ram,T/10)+Nfixcost*Nfix;
   NPP:=(GPP-Ram)/(1+rRag); 
        Rag:=rRag*NPP; 
        if Rag<0 then Rag:=0;
         Rat:=Ram+Rag;

         LFC:=rLFC*(BA+(rLFCs*(BC-BA)));
           If BC>0 then
               LFN:=BA*oqcn*BN/(oqcnA*BC)
           Else LFN:=0;
         LFN:=LFN*(rLFN-rLFNs)+(rLFNs*BN);
           If BC>0 then
               LFP:=BA*oqcp*BP/(oqcpA*BC)
           Else LFP:=0;
         LFP:=LFP*(rLFP-rLFPs)+(rLFPs*BP);
           
         WDC:=rWDC*BW;
   WDN:=WDC/qcnWD;
   WDP:=WDC/qcpWD;
   UN:=(rUN*VN*BA*ION)/(ION+KN);
   UN:=UN+Nfix;
         UP:=(rUP*VP*BA*IOP)/(IOP+KP);
          If BC>0 then
              Begin
                RC:= oqcn*BN*oqcp*BP/sqr(BC);
                RC:=power(RC,1/4);
                RC:= (LFC+Rat+WDC)*RC;
   if RC<0 then RC:=0;
                RN:=(LFN+WDN)*power((BC/(oqcn*BN)),1/2);
   if RN<0 then RN:=0;
                RP:=(LFP+WDP)*power((BC/(oqcp*BP)),1/2);
    if RP<0 then RP:=0;
              End
           Else
              Begin
                   RC:=0; RN:=0; RP:=0;
              End;
    
          WTC:=rWTC*CWDC;
          WTN:=rWTC*CWDN;
          WTP:=rWTC*CWDP;
dCWDCdt:= WDC- WTC;
           dCWDNdt:= WDN- WTN;
           dCWDPdt:= WDP- WTP;
           Phi:=power(((10*IUC)+IRC+1e-06)/(10*IRC+IUC+1e-06), VC);
           Phi:=Phi*power(((10*IUN)+IRN+ 1e-06)/(10*IRN+IUN+ 1e-06), VN);
           Phi:=Phi*power(((10*IUP)+IRP+ 1e-06)/(10*IRP+IUP+ 1e-06), VP);
          LambdaC:=((10*IUC)+IRC+1e-06)/((10*IRC)+IUC+1e-06);
          LambdaN:=((10*IUN)+IRN+1e-06)/((10*IRN)+IUN+1e-06);
LambdaP:=((10*IUP)+IRP+1e-06)/((10*IRP)+IUP+1e-06);
     



Tsoil:=T+3.13*exp(-0.5*LA);
SMI:=power(Q10soil,Tsoil/10);

ROI:=2500-tGPP/2.2;
ROI:=(2500-365*GPP/2.2)/ROI;
QC:=ROI*rQC*IOC;
QN:= ROI*rQN*ION;
QP:= ROI*rQP*IOP;
QOCI:= ROI*rQOCI*SOMCI;
QONI:= ROI*rQONI*SOMNI;
QOPI:= ROI*rQOPI*SOMPI;
QOCII:= ROI*rQOCII*SOMCII;
QONII:= ROI*rQOCII*SOMNII;
QOPII:= ROI*rQOCII*SOMPII;
DC:=rDC*SOMCI;
DN:=DC/CNPhII;
DP:=DC/CPPhII;
RhI:=SMI*rRhI*SOMCI;
RhII:= SMI*rRhII*SOMCII;    
NminI:= SMI*rNminI*SOMNI*(SOMNI/SOMCI);
PminI:= SMI*rPminI*SOMPI*(SOMPI/SOMCI);
NminII:= SMI*rRhII*SOMNII;
PminII:= SMI*rRhII*SOMPII;
IMMN:= SMI*SOMCI*(SOMCI/SOMNI)*(ION*IMMNmax)/(kIMMN+ION);
IMMP:= SMI*SOMCI*(SOMCI/SOMPI)*(IOP*IMMPmax)/(kIMMP+IOP);    
Rht:=RhI+RhII;
Re:=Rat+Rht;
NEP:=GPP-Re;
dIOCdt:=0;
dTIONdt:=Ndep+NminI+NminII -IMMN-QN;
dTIOPdt:= PW+PminI+PminII -IMMP-QP;
dSOMCIdt:=-RhI-QOCI-DC;
dSOMNIdt:=IMMN-NminI-QONI-DN;
dSOMPIdt:=IMMP-PminI-QOPI-DP;
dSOMCIIdt:=DC-RhII-QOCII;
dSOMNIIdt:=DN-NminII-QONII;
dSOMPIIdt:=DP-PminII-QOPII;
          dTIONdt:=dTIONdt-(UN-Nfix);
          dTIOPdt:=dTIOPdt-UP;
          dSOMCIdt:=dSOMCIdt+LFC+WTC;
           dSOMNIdt:=dSOMNIdt+LFN+WTN;
           dSOMPIdt:=dSOMPIdt+LFP+WTP; 
     
if CalculateDiscrete then
   begin
     {calibrate}
If (calNfix>-100) and (abs(JDay-365)<0.001) then 
   Begin
     calNfix:=0.01*exp(-2.5e-5*time); 
    
          if cumNfix > tNfix then rNfix := rNfix*(1 - calNfix)
          else                rNfix := rNfix*(1 + calNfix);
         par[FmCalculate.GetArrayIndex(vtparameter,'rNfix')].value:= rNfix;
         End;      


if (cal>-100) and (abs(JDay-365)<0.001) then
        begin
     cal:=0.01*exp(-2.5e-5*time);
          if cumDC > tDC then rDC := rDC*(1 - cal)
          else                rDC := rDC*(1 + cal);

          if cumQC > tQC then rQC := rQC*(1 - cal)
          else                rQC := rQC*(1 + cal);

          if cumQN > tQN then rQN := rQN*(1 - cal)
          else                rQN := rQN*(1 + cal);

    if cumQP > tQP then rQP := rQP*(1 - cal)
          else                rQP := rQP*(1 + cal);

    if cumQOCI > tQOCI then rQOCI := rQOCI*(1 - cal)
          else                rQOCI := rQOCI*(1 + cal);

          if cumQONI > tQONI then rQONI := rQONI*(1 - cal)
          else                rQONI := rQONI*(1 + cal);

    if cumQOPI > tQOPI then rQOPI := rQOPI*(1 - cal)
          else                rQOPI := rQOPI*(1 + cal);

    if cumQOCII > tQOCII then rQOCII := rQOCII*(1 - cal)
          else                rQOCII := rQOCII*(1 + cal);

          if cumRhI > tRhI then rRhI := rRhI*(1 - cal)
          else                rRhI := rRhI*(1 + cal);

    if cumRhII > tRhII then rRhII := rRhII*(1 - cal)
          else                rRhII := rRhII*(1 + cal);

    if cumNminI > tNminI then rNminI := rNminI*(1 - cal)
          else                rNminI := rNminI*(1 + cal);

    if cumPminI > tPminI then rPminI := rPminI*(1 - cal)
          else                rPminI := rPminI*(1 + cal);

    if cumIMMN > tIMMN then IMMNmax := IMMNmax*(1 - cal)
          else                IMMNmax := IMMNmax*(1 + cal);

    if cumIMMP > tIMMP then IMMPmax := IMMPmax*(1 - cal)
          else                IMMPmax := IMMPmax*(1 + cal);

 

          par[FmCalculate.GetArrayIndex(vtparameter,'rDC')].value:= rDC;
    par[FmCalculate.GetArrayIndex(vtparameter,'rQC')].value:= rQC;
    par[FmCalculate.GetArrayIndex(vtparameter,'rQN')].value:= rQN;
    par[FmCalculate.GetArrayIndex(vtparameter,'rQP')].value:= rQP;
          par[FmCalculate.GetArrayIndex(vtparameter,'rQOCI')].value:= rQOCI;
    par[FmCalculate.GetArrayIndex(vtparameter,'rQONI')].value:= rQONI;
    par[FmCalculate.GetArrayIndex(vtparameter,'rQOPI')].value:= rQOPI;
          par[FmCalculate.GetArrayIndex(vtparameter,'rQOCII')].value:= rQOCII;
    par[FmCalculate.GetArrayIndex(vtparameter,'rRhI')].value:= rRhI;
    par[FmCalculate.GetArrayIndex(vtparameter,'rRhII')].value:= rRhII;
    par[FmCalculate.GetArrayIndex(vtparameter,'rNminI')].value:= rNminI;
    par[FmCalculate.GetArrayIndex(vtparameter,'rPminI')].value:= rPminI;
    par[FmCalculate.GetArrayIndex(vtparameter,'IMMNmax')].value:= IMMNmax;
    par[FmCalculate.GetArrayIndex(vtparameter,'IMMPmax')].value:= IMMPmax;

        end;

if (calveg>-100) and (abs(JDay-365)<0.001) then
        begin
     calveg:=0.01*exp(-2.5e-5*time); 
     calexp:=0.01*exp(-2.5e-5*time);        
   if cumGPP > tGPP then rGPP := rGPP*(1 - calveg)
                      else                rGPP := rGPP*(1 + calveg);

if cumLFC > tLFC then rLFC := rLFC*(1 - calveg)
                      else                rLFC := rLFC*(1 + calveg);
if cumWDC > tWDC then rWDC := rWDC*(1 - calveg)
                      else                rWDC := rWDC*(1 + calveg);

if cumUN-cumNfix > tUN then rUN := rUN*(1 - calveg)
                      else                rUN := rUN*(1 + calveg);
if cumUP > tUP then rUP := rUP*(1 - calveg)
                      else                rUP := rUP*(1 + calveg);

if VN< tVN then oqcnW:=oqcnW*(1-CALEXP)
   else                 oqcnW:=oqcnW*(1+CALEXP);
if VP< tVP then oqcpW:=oqcpW*(1-CALEXP)
   else                 oqcpW:=oqcpW*(1+CALEXP);  

if cumLFN > tLFN then rLFN := rLFN*(1 - calveg)
                      else                rLFN := rLFN*(1 + calveg);
if cumLFP > tLFP then rLFP := rLFP*(1 - calveg)
                      else                rLFP := rLFP*(1 + calveg);
if cumWTC > tWTC then rWTC := rWTC*(1 - cal)
                      else                rWTC := rWTC*(1 + cal);
if cumRam > tRam then rRam := rRam*(1 - calveg)
                      else                rRam := rRam*(1 + calveg);
if cumRag > tRag then rRag := rRag*(1 - calveg)
                      else                rRag := rRag*(1 + calveg);
        

              par[FmCalculate.GetArrayIndex(vtparameter,'rGPP')].value:= rGPP; 
           
              par[FmCalculate.GetArrayIndex(vtparameter,'rLFC')].value:= rLFC; 
              par[FmCalculate.GetArrayIndex(vtparameter,'rWDC')].value:= rWDC;
                            
par[FmCalculate.GetArrayIndex(vtparameter,'rUN')].value:= rUN;
              
par[FmCalculate.GetArrayIndex(vtparameter,'rUP')].value:= rUP;

par[FmCalculate.GetArrayIndex(vtparameter,'oqcnW')].value:= oqcnW;

par[FmCalculate.GetArrayIndex(vtparameter,'oqcpW')].value:= oqcpW;

par[FmCalculate.GetArrayIndex(vtparameter,'rLFN')].value:= rLFN;
              par[FmCalculate.GetArrayIndex(vtparameter,'rLFP')].value:= rLFP;
              par[FmCalculate.GetArrayIndex(vtparameter,'rWTC')].value:= rWTC;
              par[FmCalculate.GetArrayIndex(vtparameter,'rRam')].value:= rRam;
              par[FmCalculate.GetArrayIndex(vtparameter,'rRag')].value:= rRag;
             par[FmCalculate.GetArrayIndex(vtparameter,'rNfix')].value:= rNfix;
               End;
          end;
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if (tstat[1].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBCdt := 0
else
 dBCdt:=GPP-Rat-LFC-WDC+IMG;
 
if (tstat[2].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBNdt := 0
else
 dBNdt:=UN-LFN-WDN+(IMG/oqcn);
 
if (tstat[3].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBPdt := 0
else
 dBPdt:=UP-LFP-WDP+(IMG/oqcp);
 
if (tstat[4].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVCdt := 0
else
 dVCdt:=alpha*ln(phi/ LambdaC)*VC;
 
if (tstat[5].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVNdt := 0
else
 dVNdt:=alpha*ln(phi/ LambdaN)*VN;
 
if (tstat[6].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVPdt := 0
else
 dVPdt:=alpha*ln(phi/ LambdaP)*VP;
 
if (tstat[7].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCWDCdt := 0
else
 dCWDCdt:=dCWDCdt;
 
if (tstat[8].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCWDNdt := 0
else
 dCWDNdt:=dCWDNdt;
 
if (tstat[9].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dCWDPdt := 0
else
 dCWDPdt:=dCWDPdt;
 
if (tstat[10].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIUCdt := 0
else
 dIUCdt:=sigma*(GPP-IUC);
 
if (tstat[11].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIUNdt := 0
else
 dIUNdt:=sigma*(UN-IUN);
 
if (tstat[12].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIUPdt := 0
else
 dIUPdt:=sigma*(UP-IUP);
 
if (tstat[13].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIRCdt := 0
else
 dIRCdt:=sigma*(RC-IRC);
 
if (tstat[14].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIRNdt := 0
else
 dIRNdt:=sigma*(RN-IRN);
 
if (tstat[15].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIRPdt := 0
else
 dIRPdt:=sigma*(RP-IRP);
 
if (tstat[16].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumUNdt := 0
else
 dcumUNdt:=UN;
 
if (tstat[17].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumUPdt := 0
else
 dcumUPdt:=UP;
 
if (tstat[18].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumGPPdt := 0
else
 dcumGPPdt:=GPP;
 
if (tstat[19].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumRatdt := 0
else
 dcumRatdt:=Rat;
 
if (tstat[20].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumRamdt := 0
else
 dcumRamdt:=Ram;
 
if (tstat[21].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumRagdt := 0
else
 dcumRagdt:=Rag;
 
if (tstat[22].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumNPPdt := 0
else
 dcumNPPdt:=NPP;
 
if (tstat[23].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumLFCdt := 0
else
 dcumLFCdt:=LFC;
 
if (tstat[24].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumLFNdt := 0
else
 dcumLFNdt:=LFN;
 
if (tstat[25].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumLFPdt := 0
else
 dcumLFPdt:=LFP;
 
if (tstat[26].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWDCdt := 0
else
 dcumWDCdt:=WDC;
 
if (tstat[27].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWDNdt := 0
else
 dcumWDNdt:=WDN;
 
if (tstat[28].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWDPdt := 0
else
 dcumWDPdt:=WDP;
 
if (tstat[29].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWTCdt := 0
else
 dcumWTCdt:=WTC;
 
if (tstat[30].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWTNdt := 0
else
 dcumWTNdt:=WTN;
 
if (tstat[31].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumWTPdt := 0
else
 dcumWTPdt:=WTP;
 
if (tstat[32].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumNfixdt := 0
else
 dcumNfixdt:=Nfix;
 
if (tstat[33].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dIOCdt := 0
else
 dIOCdt:=dIOCdt;
 
if (tstat[34].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dTIONdt := 0
else
 dTIONdt:=dTIONdt;
 
if (tstat[35].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dTIOPdt := 0
else
 dTIOPdt:=dTIOPdt; 
 
if (tstat[36].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMCIdt := 0
else
 dSOMCIdt:=dSOMCIdt + ManC;
 
if (tstat[37].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMNIdt := 0
else
 dSOMNIdt:=dSOMNIdt + ManN;
 
if (tstat[38].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMPIdt := 0
else
 dSOMPIdt:=dSOMPIdt + ManP;
 
if (tstat[39].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMCIIdt := 0
else
 dSOMCIIdt:=dSOMCIIdt;
 
if (tstat[40].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMNIIdt := 0
else
 dSOMNIIdt:=dSOMNIIdt;
 
if (tstat[41].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dSOMPIIdt := 0
else
 dSOMPIIdt:=dSOMPIIdt;
 
if (tstat[42].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumDCdt := 0
else
 dcumDCdt:=DC;
 
if (tstat[43].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumDNdt := 0
else
 dcumDNdt:=DN;
 
if (tstat[44].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumDPdt := 0
else
 dcumDPdt:=DP;
 
if (tstat[45].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQCdt := 0
else
 dcumQCdt:=QC; 
 
if (tstat[46].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQNdt := 0
else
 dcumQNdt:=QN;
 
if (tstat[47].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQPdt := 0
else
 dcumQPdt:=QP; 
 
if (tstat[48].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQOCIdt := 0
else
 dcumQOCIdt:=QOCI;
 
if (tstat[49].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQONIdt := 0
else
 dcumQONIdt:=QONI;
 
if (tstat[50].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQOPIdt := 0
else
 dcumQOPIdt:=QOPI;
 
if (tstat[51].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQOCIIdt := 0
else
 dcumQOCIIdt:=QOCII;
 
if (tstat[52].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQONIIdt := 0
else
 dcumQONIIdt:=QONII;
 
if (tstat[53].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumQOPIIdt := 0
else
 dcumQOPIIdt:=QOPII;
 
if (tstat[54].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumRhIdt := 0
else
 dcumRhIdt:=RhI;
 
if (tstat[55].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumRhIIdt := 0
else
 dcumRhIIdt:=RhII;
 
if (tstat[56].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumNminIdt := 0
else
 dcumNminIdt:=NminI;
 
if (tstat[57].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumNminIIdt := 0
else
 dcumNminIIdt:=NminII;
 
if (tstat[58].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumPminIdt := 0
else
 dcumPminIdt:=PminI;
 
if (tstat[59].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumPminIIdt := 0
else
 dcumPminIIdt:=PminII;
 
if (tstat[60].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumIMMNdt := 0
else
 dcumIMMNdt:=IMMN;
 
if (tstat[61].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dcumIMMPdt := 0
else
 dcumIMMPdt:=IMMP;
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put the state variables back into the global arrays in case the state variable
  was manually changed in this procedure (e.g. discrete state variables or steady state
  calculations).   }
tstat[1].value := BC;
tstat[2].value := BN;
tstat[3].value := BP;
tstat[4].value := VC;
tstat[5].value := VN;
tstat[6].value := VP;
tstat[7].value := CWDC;
tstat[8].value := CWDN;
tstat[9].value := CWDP;
tstat[10].value := IUC;
tstat[11].value := IUN;
tstat[12].value := IUP;
tstat[13].value := IRC;
tstat[14].value := IRN;
tstat[15].value := IRP;
tstat[16].value := cumUN;
tstat[17].value := cumUP;
tstat[18].value := cumGPP;
tstat[19].value := cumRat;
tstat[20].value := cumRam;
tstat[21].value := cumRag;
tstat[22].value := cumNPP;
tstat[23].value := cumLFC;
tstat[24].value := cumLFN;
tstat[25].value := cumLFP;
tstat[26].value := cumWDC;
tstat[27].value := cumWDN;
tstat[28].value := cumWDP;
tstat[29].value := cumWTC;
tstat[30].value := cumWTN;
tstat[31].value := cumWTP;
tstat[32].value := cumNfix;
tstat[33].value := IOC;
tstat[34].value := TION;
tstat[35].value := TIOP;
tstat[36].value := SOMCI;
tstat[37].value := SOMNI;
tstat[38].value := SOMPI;
tstat[39].value := SOMCII;
tstat[40].value := SOMNII;
tstat[41].value := SOMPII;
tstat[42].value := cumDC;
tstat[43].value := cumDN;
tstat[44].value := cumDP;
tstat[45].value := cumQC;
tstat[46].value := cumQN;
tstat[47].value := cumQP;
tstat[48].value := cumQOCI;
tstat[49].value := cumQONI;
tstat[50].value := cumQOPI;
tstat[51].value := cumQOCII;
tstat[52].value := cumQONII;
tstat[53].value := cumQOPII;
tstat[54].value := cumRhI;
tstat[55].value := cumRhII;
tstat[56].value := cumNminI;
tstat[57].value := cumNminII;
tstat[58].value := cumPminI;
tstat[59].value := cumPminII;
tstat[60].value := cumIMMN;
tstat[61].value := cumIMMP;

{  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dBCdt;
tproc[2].value := dBNdt;
tproc[3].value := dBPdt;
tproc[4].value := dVCdt;
tproc[5].value := dVNdt;
tproc[6].value := dVPdt;
tproc[7].value := dCWDCdt;
tproc[8].value := dCWDNdt;
tproc[9].value := dCWDPdt;
tproc[10].value := dIUCdt;
tproc[11].value := dIUNdt;
tproc[12].value := dIUPdt;
tproc[13].value := dIRCdt;
tproc[14].value := dIRNdt;
tproc[15].value := dIRPdt;
tproc[16].value := dcumUNdt;
tproc[17].value := dcumUPdt;
tproc[18].value := dcumGPPdt;
tproc[19].value := dcumRatdt;
tproc[20].value := dcumRamdt;
tproc[21].value := dcumRagdt;
tproc[22].value := dcumNPPdt;
tproc[23].value := dcumLFCdt;
tproc[24].value := dcumLFNdt;
tproc[25].value := dcumLFPdt;
tproc[26].value := dcumWDCdt;
tproc[27].value := dcumWDNdt;
tproc[28].value := dcumWDPdt;
tproc[29].value := dcumWTCdt;
tproc[30].value := dcumWTNdt;
tproc[31].value := dcumWTPdt;
tproc[32].value := dcumNfixdt;
tproc[33].value := dIOCdt;
tproc[34].value := dTIONdt;
tproc[35].value := dTIOPdt;
tproc[36].value := dSOMCIdt;
tproc[37].value := dSOMNIdt;
tproc[38].value := dSOMPIdt;
tproc[39].value := dSOMCIIdt;
tproc[40].value := dSOMNIIdt;
tproc[41].value := dSOMPIIdt;
tproc[42].value := dcumDCdt;
tproc[43].value := dcumDNdt;
tproc[44].value := dcumDPdt;
tproc[45].value := dcumQCdt;
tproc[46].value := dcumQNdt;
tproc[47].value := dcumQPdt;
tproc[48].value := dcumQOCIdt;
tproc[49].value := dcumQONIdt;
tproc[50].value := dcumQOPIdt;
tproc[51].value := dcumQOCIIdt;
tproc[52].value := dcumQONIIdt;
tproc[53].value := dcumQOPIIdt;
tproc[54].value := dcumRhIdt;
tproc[55].value := dcumRhIIdt;
tproc[56].value := dcumNminIdt;
tproc[57].value := dcumNminIIdt;
tproc[58].value := dcumPminIdt;
tproc[59].value := dcumPminIIdt;
tproc[60].value := dcumIMMNdt;
tproc[61].value := dcumIMMPdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := GPP;
tproc[ModelDef.numstate + 2].value := Ram;
tproc[ModelDef.numstate + 3].value := Rag;
tproc[ModelDef.numstate + 4].value := Rat;
tproc[ModelDef.numstate + 5].value := NPP;
tproc[ModelDef.numstate + 6].value := UN;
tproc[ModelDef.numstate + 7].value := UP;
tproc[ModelDef.numstate + 8].value := dum1;
tproc[ModelDef.numstate + 9].value := dum2;
tproc[ModelDef.numstate + 10].value := oqcn;
tproc[ModelDef.numstate + 11].value := oqcp;
tproc[ModelDef.numstate + 12].value := phi;
tproc[ModelDef.numstate + 13].value := LambdaC;
tproc[ModelDef.numstate + 14].value := LambdaN;
tproc[ModelDef.numstate + 15].value := LambdaP;
tproc[ModelDef.numstate + 16].value := LFC;
tproc[ModelDef.numstate + 17].value := LFN;
tproc[ModelDef.numstate + 18].value := LFP;
tproc[ModelDef.numstate + 19].value := WDC;
tproc[ModelDef.numstate + 20].value := WDN;
tproc[ModelDef.numstate + 21].value := WDP;
tproc[ModelDef.numstate + 22].value := Nupcost;
tproc[ModelDef.numstate + 23].value := Nfix;
tproc[ModelDef.numstate + 24].value := RC;
tproc[ModelDef.numstate + 25].value := RN;
tproc[ModelDef.numstate + 26].value := RP;
tproc[ModelDef.numstate + 27].value := BA;
tproc[ModelDef.numstate + 28].value := BW;
tproc[ModelDef.numstate + 29].value := LA;
tproc[ModelDef.numstate + 30].value := LAT2;
tproc[ModelDef.numstate + 31].value := RL;
tproc[ModelDef.numstate + 32].value := dum5;
tproc[ModelDef.numstate + 33].value := dum6;
tproc[ModelDef.numstate + 34].value := WTC;
tproc[ModelDef.numstate + 35].value := WTN;
tproc[ModelDef.numstate + 36].value := WTP;
tproc[ModelDef.numstate + 37].value := DC;
tproc[ModelDef.numstate + 38].value := DN;
tproc[ModelDef.numstate + 39].value := DP;
tproc[ModelDef.numstate + 40].value := QC;
tproc[ModelDef.numstate + 41].value := QN;
tproc[ModelDef.numstate + 42].value := QP;
tproc[ModelDef.numstate + 43].value := QOCI;
tproc[ModelDef.numstate + 44].value := QONI;
tproc[ModelDef.numstate + 45].value := QOPI;
tproc[ModelDef.numstate + 46].value := QOCII;
tproc[ModelDef.numstate + 47].value := QONII;
tproc[ModelDef.numstate + 48].value := QOPII;
tproc[ModelDef.numstate + 49].value := RhI;
tproc[ModelDef.numstate + 50].value := RhII;
tproc[ModelDef.numstate + 51].value := NminI;
tproc[ModelDef.numstate + 52].value := NminII;
tproc[ModelDef.numstate + 53].value := PminI;
tproc[ModelDef.numstate + 54].value := PminII;
tproc[ModelDef.numstate + 55].value := IMMN;
tproc[ModelDef.numstate + 56].value := IMMP;
tproc[ModelDef.numstate + 57].value := ION;
tproc[ModelDef.numstate + 58].value := IOP;
tproc[ModelDef.numstate + 59].value := Jday;
tproc[ModelDef.numstate + 60].value := Tsoil;
tproc[ModelDef.numstate + 61].value := SMI;
tproc[ModelDef.numstate + 62].value := ROI;
tproc[ModelDef.numstate + 63].value := calib;
tproc[ModelDef.numstate + 64].value := calibA;
tproc[ModelDef.numstate + 65].value := calibB;
tproc[ModelDef.numstate + 66].value := aqcn;
tproc[ModelDef.numstate + 67].value := aqcp;
tproc[ModelDef.numstate + 68].value := NEP;
tproc[ModelDef.numstate + 69].value := Rht;
tproc[ModelDef.numstate + 70].value := Re;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t, drt, tdrive, tpar, tempstate, tempproc, false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
